%**********************************************************
% RALF - Reinforced Active Learning Formulation
%**********************************************************
% S. Ebert, M. Fritz, B. Schiele
% RALF: A Reinforced Active Learning Formulation for Object Class
% Recognition, CVPR 2012
%
% The authors and copyright holders are S. Ebert, M. Fritz, and B. Schiele.
% You might redistribute, reuse, and mofidy this code for research 
% purposes. Any commercial application needs a written agreement by one of
% the authors.

xx = 1:obj.numtrain;
switch obj.betaFun
    case 'x'
        obj.betas = xx;
        obj.betas = (obj.betas-min(obj.betas))*(1-0)/(max(obj.betas)-min(obj.betas));
    case 'x25-75'
        obj.betas = xx;
        obj.betas = (obj.betas-min(obj.betas))*(0.75-0.25)/(max(obj.betas)-min(obj.betas))+0.25;
    case 'x25-100'
        obj.betas = xx;
        obj.betas = (obj.betas-min(obj.betas))*(1-0.25)/(max(obj.betas)-min(obj.betas))+0.25;
    case 'x0-75'
        obj.betas = xx;
        obj.betas = (obj.betas-min(obj.betas))*(0.75-0)/(max(obj.betas)-min(obj.betas));
    case '-x'
        obj.betas = -xx;
        obj.betas = (obj.betas-min(obj.betas))*(1-0)/(max(obj.betas)-min(obj.betas));
    case '-x100-25'
        obj.betas = -xx;
        obj.betas = (obj.betas-min(obj.betas))*(1.0-0.25)/(max(obj.betas)-min(obj.betas))+0.25;
    case '-x75-25'
        obj.betas = -xx;
        obj.betas = (obj.betas-min(obj.betas))*(0.75-0.25)/(max(obj.betas)-min(obj.betas))+0.25;
    case '-x75-0'
        obj.betas = -xx;
        obj.betas = (obj.betas-min(obj.betas))*(0.75-0)/(max(obj.betas)-min(obj.betas));
    case 'logx25-100'
        obj.betas = log(xx);
        obj.betas = (obj.betas-min(obj.betas))*(1-0.25)/(max(obj.betas)-min(obj.betas))+0.25;
    case 'logx25-75'
        obj.betas = log(xx);
        obj.betas = (obj.betas-min(obj.betas))*(0.75-0.25)/(max(obj.betas)-min(obj.betas))+0.25;
    case 'logx0-75'
        obj.betas = log(xx);
        obj.betas = (obj.betas-min(obj.betas))*(0.75-0)/(max(obj.betas)-min(obj.betas));
    case 'log(x)'
        obj.betas = log(xx);
        obj.betas = (obj.betas-min(obj.betas))*(1-0)/(max(obj.betas)-min(obj.betas));
    case 'x^2'
        obj.betas = xx.^2;
        obj.betas = (obj.betas-min(obj.betas))*(1-0)/(max(obj.betas)-min(obj.betas));
    case '0.0'
        obj.betas = ones(1,length(xx)).*0;
    case'poly0-50'
        p = polyfit([1 round(numtrain/2) numtrain], [0 0.5 0], 2);
        obj.betas = polyval(p, xx);
    case'poly0-75'
        p = polyfit([1 round(numtrain/2) numtrain], [0 0.75 0], 2);
        obj.betas = polyval(p, xx);
    case'poly25-50'
        p = polyfit([1 round(numtrain/2) numtrain], [0.25 0.5 0.25], 2);
        obj.betas = polyval(p, xx);
    case'poly25-75'
        p = polyfit([1 round(numtrain/2) numtrain], [0.25 0.75 0.25], 2);
        obj.betas = polyval(p, xx);
    case '0.25'
        obj.betas = ones(1,length(xx)).*0.25;
    case '0.5'
        obj.betas = ones(1,length(xx)).*0.5;
    case '0.75'
        obj.betas = ones(1,length(xx)).*0.75;
    case '0.1'
        obj.betas = ones(1,length(xx)).*0.1;
    case '0.9'
        obj.betas = ones(1,length(xx)).*0.9;
    case '1.0'
        obj.betas = ones(1,length(xx)).*1.0;
    case '-log(x)'
        obj.betas = -log(xx);
        obj.betas = (obj.betas-min(obj.betas))*(1-0)/(max(obj.betas)-min(obj.betas));
    case '-x^2'
        obj.betas = -xx.^2;
        obj.betas = (obj.betas-min(obj.betas))*(1-0)/(max(obj.betas)-min(obj.betas));
    case '-x^2100-25'
        obj.betas = -xx.^2;
        obj.betas = (obj.betas-min(obj.betas))*(1-0.25)/(max(obj.betas)-min(obj.betas))+0.25;
    case '-x^275-25'
        obj.betas = -xx.^2;
        obj.betas = (obj.betas-min(obj.betas))*(0.75-0.25)/(max(obj.betas)-min(obj.betas))+0.25;
    case '-x^275-0'
        obj.betas = -xx.^2;
        obj.betas = (obj.betas-min(obj.betas))*(0.75-0)/(max(obj.betas)-min(obj.betas));
    case 'sin(x)'
        obj.betas = sin((xx-min(xx))*(pi/(max(xx)-min(xx))));
end
clear xx;
