%**********************************************************
% RALF - Reinforced Active Learning Formulation
%**********************************************************
% S. Ebert, M. Fritz, B. Schiele
% RALF: A Reinforced Active Learning Formulation for Object Class
% Recognition, CVPR 2012
%
% The authors and copyright holders are S. Ebert, M. Fritz, and B. Schiele.
% You might redistribute, reuse, and mofidy this code for research 
% purposes. Any commercial application needs a written agreement by one of
% the authors.

function obj = ConstructkNNGraph(obj)

%computes the distance matrix
switch obj.distance
    case {'L2'}
        W = squareform(pdist(obj.data));
        
    case {'L1', 'cityblock'}
        W = squareform(pdist(obj.data, 'cityblock'));
        
    case {'cos'}
        W = squareform(pdist(obj.data, 'cosine'));
        
    case {'corr'}
        W = squareform(pdist(obj.data, 'correlation'));
        
end;


%sorts matrix row by row
[sval sidx] = sort(W,2);
%ensures that at least k-NN are possible
k= min(obj.kLP+1, size(W,1)-1);
%constructs a sparse graph with the distance values
obj.Graph = sparse(repmat(sidx(:,1),1,k), sidx(:,2:k+1), sval(:,2:k+1));

%weighted graph construction
if isfield(obj, 'isWeighted') && (obj.isWeighted==0)
    %heuristic to calculate sigma: uses the k-NN to find a good weighting
    if ~isfield(obj, 'sigma')
        obj.sigma = mean(mean(obj.Graph));
    end
    %Gaussian weighting
    obj.Graph = exp(-obj.Graph./(2*obj.sigma));
else
    obj.Graph = obj.Graph > 0;
end

%symmetric graph construction
if isfield(obj, 'isSymmetric') && (obj.isSymmetric==1)
    %edges that are bidirectional get a much higher weight (w_ij<=2)
    obj.Graph = obj.Graph + obj.Graph';
end;
