%**********************************************************
% RALF - Reinforced Active Learning Formulation
%**********************************************************
% S. Ebert, M. Fritz, B. Schiele
% RALF: A Reinforced Active Learning Formulation for Object Class
% Recognition, CVPR 2012
%
% The authors and copyright holders are S. Ebert, M. Fritz, and B. Schiele.
% You might redistribute, reuse, and mofidy this code for research 
% purposes. Any commercial application needs a written agreement by one of
% the authors.

function obj = KNN(obj)
%
% KNN implementation; 1-NN is used for classifcation as default because it 
% always shows best performance and to compute the score we k=10 otherwise
% it is not applicable with entropy-based or margin-based sampling
%

% number of neighbors for classification
neighbors = 1;
% number of neighbors to compute the score for entropy/margin criteria
numK = max(min(10,obj.numTrain-5), neighbors);

K = obj.kernel([obj.traininds;obj.testinds], [obj.traininds;obj.testinds]);
[sval sidx] = sort(K(:,1:obj.numTrain), 2);
obj.scores = full(sparse(repmat((1:obj.numData)',1,numK), ...
    obj.trainlabels(sidx(:, 1:numK)),double(sval(:,1:numK))));
if neighbors < 10
    temp = full (sparse(repmat((1:obj.numData)', 1,neighbors), ...
        obj.trainlabels(sidx(:,1:neighbors)),1));
else
    temp = obj.scores;
end

[mval obj.predIdx] = max(temp, [], 2);
