%**********************************************************
% RALF - Reinforced Active Learning Formulation
%**********************************************************
% S. Ebert, M. Fritz, B. Schiele
% RALF: A Reinforced Active Learning Formulation for Object Class
% Recognition, CVPR 2012
%
% The authors and copyright holders are S. Ebert, M. Fritz, and B. Schiele.
% You might redistribute, reuse, and mofidy this code for research 
% purposes. Any commercial application needs a written agreement by one of
% the authors.

function obj = LPZhou(obj)

%symmetric normalized graph laplacian
G = obj.Graph([obj.traininds;obj.testinds], [obj.traininds;obj.testinds]);
D = sparse(1:length(G), 1:length(G), (sum(G,2)+eps).^(-1/2));
S = D*G*D;

%one-vs-all classification
%one classification vector for each class
Y0 = zeros(obj.numData,obj.numClasses);
Y0(1:obj.numTrain,:) = -1;
for i=1:obj.numClasses
    Y0(obj.trainlabels==i,i) = 1;
end
Yold = Y0;

%iteration counter
count = 1;
%difference to abort if it is converged
diff = 1;

if obj.maxIterLP==0
    %closed form solution if max number of iterations is set to zero
    %slow and tends to overfit
    Ynew = inv(eye(obj.numData)-obj.alphaLP*S)*Y0;
else
    while(diff > 10^-3 && count < obj.maxIterLP),
        %iterative propagation; 
        %faster in particular for large matrices
        Ynew = obj.alphaLP*S*Yold + (1-obj.alphaLP)*Y0;
        diff = sum(sum(abs(Yold - Ynew)));
        Yold = Ynew;
        count = count + 1;
    end
end

%full score matrix required for entropy-based and margin-based sampling
obj.scores = Ynew;

%final prediction and the respective score
[obj.predVal obj.predIdx] = max(Ynew,[],2);
