function obj = LPZhu(obj)
%
%Zhu ICML 2003
%Semi-supervised learning using Gaussian fields and harmonic functions
%

% generates the random walk matrix P for propagation
W = obj.Graph([obj.traininds;obj.testinds], [obj.traininds;obj.testinds]);
P = bsxfun(@rdivide, W, sum(W,2));

%one-vs-all classification
%one classification vector for each class
Ybin = zeros(obj.numData,obj.numClasses);
Ybin(1:obj.numTrain,:) = -1;
for i=1:obj.numClasses
    Ybin(obj.trainlabels==i,i) = 1;
end

% initialize prediction matrix
Ynew = zeros(obj.numData, obj.numClasses);

for c=1:obj.numClasses
    Y0 = Ybin(:,c);
    Y = Y0;
    %iteration counter
    count = 1;
    %difference to abort if it is converged
    diff = 1;
    while(diff > 10^-3 && count < obj.maxIterLP)
        % saves previous state for the convergence criteria
        Yprev = Y;
        % propagation
        Y = P*Y;
        % reset predictions of labeled data by the original label
        Y(Ybin(:,c)~=0) = Y0(Ybin(:,c)~=0);
        % difference for convergence
        diff = full(sum(abs(Y-Yprev)));
        count = count + 1;
    end;
    Y(isnan(Y)) = 0;
    Ynew(:,c) = Y;
end;

%full score matrix required for entropy-based and margin-based sampling
obj.scores = Ynew;

%final prediction and the respective score
[obj.predVal obj.predIdx] = max(Ynew,[],2);