%**********************************************************
% RALF - Reinforced Active Learning Formulation
%**********************************************************
% S. Ebert, M. Fritz, B. Schiele
% RALF: A Reinforced Active Learning Formulation for Object Class
% Recognition, CVPR 2012
%
% The authors and copyright holders are S. Ebert, M. Fritz, and B. Schiele.
% You might redistribute, reuse, and mofidy this code for research 
% purposes. Any commercial application needs a written agreement by one of
% the authors.

function runBetaEstimation
%
% run previous methods published by [Osugi2005] including our novel 
% improvements for this methods
%
% [Osugi2005] - T. Osugi, S. Scott. Balancing Exploration and Exploitation:
% A New Algorithm for Active Machine Learning. In ICDM, 2005.
% 
% This method estimate only the trade-off beta between exploration and
% exploitation given by the feedback from the previous iteration. There is
% no learning involved and it is also not possible to switch among more
% than 2 criteria.
%
% Our improvements address a the scaling of this feedback as well as the
% feedback function itself.

%change dataset: eth80 or cpascal
obj.dataset = 'eth80';

%% Beta estimation parameter

% use beta as a probability to switch randomly between exploration and
% exploitation [Osugi2005] or use it as a trade-off between both criteria
obj.betaAsProb = 1;

% scaling function proposed by [Osugi2005] or our novel flexible scaling 
% function (false)
obj.origFeedbackScaling = 1;

% feedback measure: 1) change between previous and current hypothesis Y
% [Osugi2005], 2) overall entropy of the class posterior, 3) overall margin
% between best and second best class posterior
feedback = {'HypothesisChange', 'Entropy'};
obj.feedbackMeasure = str2func(feedback{1});

%run beta estimation with
BetaEstimation(obj);