%**********************************************************
% RALF - Reinforced Active Learning Formulation
%**********************************************************
% S. Ebert, M. Fritz, B. Schiele
% RALF: A Reinforced Active Learning Formulation for Object Class
% Recognition, CVPR 2012
%
% The authors and copyright holders are S. Ebert, M. Fritz, and B. Schiele.
% You might redistribute, reuse, and mofidy this code for research 
% purposes. Any commercial application needs a written agreement by one of
% the authors.

function runCriteriaAnalysis

% fixed and time-varying combination (see Sec. 6, Tab. 2) without any 
% reinforcement learning; 
% obj.isRandomSampling = true executes random sampling with a uniform 
% distribution

%For the single criteria use betaFun=0.0 (pure exploration) or betaFun=1.0
%(pure exploitation). 
%Note for pure exploration there are only minor differences among the 
%different runs because of the fixed pre-computed values. 

%change dataset: eth80 or cpascal
obj.dataset ='eth80';

%initialize all parameters and settings
init;

%beta functions from the paper
funcs = {'0.5', 'log(x)', 'x', '-log(x)', '-x'};
for f= 1:length(funcs)
    obj.betaFun = char(funcs{f});
    iter = 1;
    %exploitation criteria
    for inf = {'MarginBasedInf', 'EntropyBasedInf'}
        obj.exploit = char(inf);
        %exploration criteria
        for rep = {'NodePotentialRep', 'KernelFarthestRep', 'GraphDensityBasedRep'}
            obj.explore = char(rep);
            for run =1:obj.maxRun
                resetValues;
                fprintf('\nrun %d:\n', run);
                obj.traininds = [];
                
                while length(obj.traininds) < obj.numtrain
                    %sampling of 
                    obj = Sampling(obj);
                    obj = GetClassificationResults(obj);
                    if mod(obj.numTrain,10)==0
                        fprintf('%s+%s with %s, iter %d, acc %2.1f, avgprec %2.1f\n', ...
                            obj.exploit(1:3), obj.explore(1:3), obj.betaFun, ...
                            length(obj.traininds), obj.acc, obj.avgprec);
                    end
                end
                results(iter,f,run) = obj.acc;
            end
            
            mean(results,3)
            iter = iter+1;
        end
    end
end
