%**********************************************************
% RALF - Reinforced Active Learning Formulation
%**********************************************************
% S. Ebert, M. Fritz, B. Schiele
% RALF: A Reinforced Active Learning Formulation for Object Class
% Recognition, CVPR 2012
%
% The authors and copyright holders are S. Ebert, M. Fritz, and B. Schiele.
% You might redistribute, reuse, and mofidy this code for research
% purposes. Any commercial application needs a written agreement by one of
% the authors.

function BetaEstimation(obj)
init;

% init beta
obj.betas = zeros(obj.numClasses,1);
for run =1:obj.maxRun
    % probability to select exploration or the amount of exploration
    prob = 0.5;
    resetValues;
    fprintf('\nrun %d:\n', run);
    obj.traininds = [];
    
    while length(obj.traininds) < obj.numtrain
        obj = Sampling(obj);
        it = length(obj.traininds)+1-obj.numClasses;
        obj = GetClassificationResults(obj);
        
        if it > obj.numClasses
            %get feedback
            obj = obj.feedbackMeasure(obj);
            
            % scales the feedback into the the range [-1,1]
            % otherwise it is almost always positive resulting in a
            % bias to exploration
            if obj.origFeedbackScaling
                % scaling function from [Osugi2005]
                obj.feedback = 3-4*obj.feedback;
            else
                % scales these values depending on the entire
                % sequence of feedbacks from the first iteration to
                % the current iteration
                feedbackSeq(it-obj.numClasses) = obj.feedback;
                obj.feedback = (obj.feedback-min(feedbackSeq))*...
                    2/(max(feedbackSeq)-min(feedbackSeq))-1;
            end
            
            % updates the current probability based on the current
            % feedback
            prob = max(min(obj.betaEstLambda*prob*...
                exp(obj.feedback),1-obj.betaEstEps),obj.betaEstEps);
            
            % if activated prob is used to select with probability
            % prob pure exploration (0) otherwise it is used as a
            % trade-off between both criteria
            if obj.betaAsProb
                if rand<=prob
                    obj.betas(it+1) = 0;
                else
                    obj.betas(it+1) = 1;
                end
            else
                obj.betas(it+1) = 1-prob;
            end
        end
        obj.prevScores = obj.scores;
        if mod(obj.numTrain,10)==0
            fprintf('%s+%s with beta %2.1f, iter %d, acc %2.1f, prob %1.2f\n', ...
                obj.exploit(1:3), obj.explore(1:3), obj.betas(it), ...
                length(obj.traininds), obj.acc, prob);
        end
    end
    results(run) = obj.acc;
end
mean(results)