%**********************************************************
% RALF - Reinforced Active Learning Formulation
%**********************************************************
% S. Ebert, M. Fritz, B. Schiele
% RALF: A Reinforced Active Learning Formulation for Object Class
% Recognition, CVPR 2012
%
% The authors and copyright holders are S. Ebert, M. Fritz, and B. Schiele.
% You might redistribute, reuse, and mofidy this code for research
% purposes. Any commercial application needs a written agreement by one of
% the authors.

function obj = RALF(obj)
%
% runs our RALF based on a Markov Decision Process (MDP) that considers the
% entire active learning sequence as a decision process and learns online a
% weighting of each (state,action) pair
%

init;
for run =1:obj.maxRun
    
    fprintf('\nrun %d:\n', run);
    obj.traininds = [];
    % initialize the MDP
    obj = initMDP(obj);
    
    while length(obj.traininds) < obj.numtrain
        % sampling of the next label
        obj = Sampling(obj);
        it = length(obj.traininds)+1-obj.numClasses;
        
        % classification based on the current labels
        obj = GetClassificationResults(obj);
        
        % updates the Q table given the current feedback
        obj = updateQTable(obj);
        
        % get the next state in our MDP
        obj = getNextStateMDP(obj);
        if mod(obj.numTrain,10)==0
            fprintf('state %s+%s with action beta=%1.2f, iter %d, acc %2.1f\n', ...
                obj.exploit(1:3), obj.explore(1:3),  ...
                obj.betas(it), length(obj.traininds), obj.acc);
        end
    end
    results(run) = obj.acc;
end
mean(results)