%**********************************************************
% RALF - Reinforced Active Learning Formulation
%**********************************************************
% S. Ebert, M. Fritz, B. Schiele
% RALF: A Reinforced Active Learning Formulation for Object Class
% Recognition, CVPR 2012
%
% The authors and copyright holders are S. Ebert, M. Fritz, and B. Schiele.
% You might redistribute, reuse, and mofidy this code for research
% purposes. Any commercial application needs a written agreement by one of
% the authors.

function obj = getNextStateMDP(obj)
%
% draw the next (first) state in the MDP: first iterations are randomly
% assigned and after that
%
it = length(obj.traininds);
if it < obj.numClasses+20
    actIdx = find(~isnan(obj.Q(obj.stateSeq(it),:)));
    nextAction = ceil(rand*length(actIdx));
    obj.actionSeq(it) = actIdx(nextAction);
else
    % permutates the ordering of the states
    %permu = randperm(size(Q,2));
    [mval midx] = max(obj.Q(obj.stateSeq(it),:));
    obj.actionSeq(it) = midx;
end

% state switch if action=0
if obj.actions(obj.actionSeq(it))==0
    % number of next action corresponds to next state; case of selecting
    % the same state is avoided by using nan in the diagonal
    obj.stateSeq(it+1) = obj.actionSeq(it);
    
    % change criteria according to the new state
    obj.exploit = char(obj.criteria{obj.states(obj.stateSeq(it+1),1)});
    obj.explore = char(obj.criteria{obj.states(obj.stateSeq(it+1),2)});
else
    obj.stateSeq(it+1) = obj.stateSeq(it);
end

% next beta value is assigned depending on the action. It is set to 0.5 if
% there is a state switch.
if obj.actions(obj.actionSeq(it))==0
    obj.betas(it+1-obj.numClasses) = 0.5;
else
    obj.betas(it+1-obj.numClasses) = obj.tradeoffs(obj.actionSeq(it)- ...
        sum(obj.actions==0));
end
