%**********************************************************
% RALF - Reinforced Active Learning Formulation
%**********************************************************
% S. Ebert, M. Fritz, B. Schiele
% RALF: A Reinforced Active Learning Formulation for Object Class
% Recognition, CVPR 2012
%
% The authors and copyright holders are S. Ebert, M. Fritz, and B. Schiele.
% You might redistribute, reuse, and mofidy this code for research
% purposes. Any commercial application needs a written agreement by one of
% the authors.

function obj = initMDP(obj)
% -------
% initialization of the Markov Decision Process, i.e., states and actions
% -------

% set a flag for each criteria if it is an exploration (= representative)
% sampling criteria. This information is required as each state is a
% combination of one exploration and one exploitation criteria.
obj.isExplore = zeros(length(obj.criteria),1);
for i=1:length(obj.criteria)
    if strfind(obj.criteria{i}, 'Rep')
        obj.isExplore(i) = 1;
    end
end

% initialization of the state space: numExploration*numExploitation x 2;
% first column contains the index to the exploitation criteria and second
% column to the exploration criteria
infIdx = find(obj.isExplore==0);
repIdx = find(obj.isExplore==1);
count = 1;
for i=1:length(infIdx)
    for j=1:length(repIdx)
        obj.states(count,:) = [infIdx(i) repIdx(j)];
        count = count + 1;
    end
end
numStates = size(obj.states,1);

% initialize action types: 0 (switch among different states), 1 (switch
% among different trade-offs)
% vector of size 1xnumStates+numBetas
obj.actions = [zeros(1,numStates) ones(1,length(obj.tradeoffs))];

% initialize Q table
numActions = length(obj.actions);
obj.Q = zeros(numStates, numActions);
%diagonal is set to nan to avoid a loop
if numStates > 1
    obj.Q(1:numStates, 1:numStates) = diag(diag(obj.Q)*inf);
else
    obj.Q(1,1) = nan;
end

% assign first state randomly
obj.stateSeq(obj.numClasses) = ceil(rand*numStates);

% assign sampling criteria according to the state
obj.exploit = char(obj.criteria{obj.states(obj.stateSeq(obj.numClasses),1)});
obj.explore = char(obj.criteria{obj.states(obj.stateSeq(obj.numClasses),2)});
obj.betas(obj.numClasses) = 0.5;